﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Configuration;
using System.ComponentModel;
using System.Windows;
using System.Windows.Data;
using Telerik.Windows.Controls;
using Telerik.Windows.Data;
using Telerik.Windows.Controls.GridView;
using System.Windows.Controls;
using System.Runtime.Serialization;
using System.IO;
using System.IO.IsolatedStorage;

namespace Telerik.Settings
{
    public class RadGridViewSettings
    {
        public RadGridViewSettings()
        {
            //
        }

        public class RadGridViewApplicationSettings : Dictionary<string, object>
        {
            private RadGridViewSettings settings;

            private DataContractSerializer serializer = null;

            public RadGridViewApplicationSettings()
            {
                //
            }

            public RadGridViewApplicationSettings(RadGridViewSettings settings)
            {
                this.settings = settings;
                this.serializer = new DataContractSerializer(typeof(RadGridViewApplicationSettings),
                            new[] 
                            { 
                                typeof(List<ColumnSetting>), 
                                typeof(List<FilterSetting>), 
                                typeof(List<GroupSetting>),
                                typeof(List<SortSetting>), 
                                typeof(List<PropertySetting>)
                            });
            }

            public string PersistID
            {
                get
                {
                    if (!ContainsKey("PersistID") && settings.grid != null)
                    {
                        this["PersistID"] = settings.grid.Name;
                    }

                    return (string) this["PersistID"];
                }
            }

            public int FrozenColumnCount
            {
                get
                {
                    if (!ContainsKey("FrozenColumnCount"))
                    {
                        this["FrozenColumnCount"] = 0;
                    }

                    return (int) this["FrozenColumnCount"];
                }
                set
                {
                    this["FrozenColumnCount"] = value;
                }
            }

            public List<ColumnSetting> ColumnSettings
            {
                get
                {
                    if (!ContainsKey("ColumnSettings"))
                    {
                        this["ColumnSettings"] = new List<ColumnSetting>();
                    }

                    return (List<ColumnSetting>) this["ColumnSettings"];
                }
            }

            public List<SortSetting> SortSettings
            {
                get
                {
                    if (!ContainsKey("SortSettings"))
                    {
                        this["SortSettings"] = new List<SortSetting>();
                    }

                    return (List<SortSetting>) this["SortSettings"];
                }
            }

            public List<GroupSetting> GroupSettings
            {
                get
                {
                    if (!ContainsKey("GroupSettings"))
                    {
                        this["GroupSettings"] = new List<GroupSetting>();
                    }

                    return (List<GroupSetting>) this["GroupSettings"];
                }
            }

            public List<FilterSetting> FilterSettings
            {
                get
                {
                    if (!ContainsKey("FilterSettings"))
                    {
                        this["FilterSettings"] = new List<FilterSetting>();
                    }

                    return (List<FilterSetting>) this["FilterSettings"];
                }
            }

            public void Reload()
            {
                try
                {
                    using (IsolatedStorageFile file = IsolatedStorageFile.GetUserStoreForApplication())
                    {
                        using (IsolatedStorageFileStream stream = new IsolatedStorageFileStream(PersistID, FileMode.Open, file))
                        {
                            if (stream.Length > 0)
                            {
                                RadGridViewApplicationSettings loaded = (RadGridViewApplicationSettings) serializer.ReadObject(stream);

                                FrozenColumnCount = loaded.FrozenColumnCount;

                                ColumnSettings.Clear();
                                loaded.ColumnSettings.ForEach(cs => ColumnSettings.Add(cs));

                                FilterSettings.Clear();
                                loaded.FilterSettings.ForEach(fs => FilterSettings.Add(fs));

                                GroupSettings.Clear();
                                loaded.GroupSettings.ForEach(gs => GroupSettings.Add(gs));

                                SortSettings.Clear();
                                loaded.SortSettings.ForEach(ss => SortSettings.Add(ss));
                            }
                        }
                    }
                }
                catch
                { 
                    
                }
            }

            public void Reset()
            {
                using (IsolatedStorageFile file = IsolatedStorageFile.GetUserStoreForApplication())
                {
                    try
                    {
                        file.DeleteFile(PersistID);
                    }
                    catch
                    {
                        //
                    }
                }
            }

            public void Save()
            {
                using (IsolatedStorageFile file = IsolatedStorageFile.GetUserStoreForApplication())
                {
                    using (IsolatedStorageFileStream stream = new IsolatedStorageFileStream(PersistID, FileMode.Create, file))
                    {
                        serializer.WriteObject(stream, this);
                    }
                }
            }
        }

        private RadGridView grid = null;

        public RadGridViewSettings(RadGridView grid)
        {
            this.grid = grid;
        }

        public static readonly DependencyProperty IsEnabledProperty
           = DependencyProperty.RegisterAttached("IsEnabled", typeof(bool), typeof(RadGridViewSettings),
                new PropertyMetadata(new PropertyChangedCallback(OnIsEnabledPropertyChanged)));

        public static void SetIsEnabled(DependencyObject dependencyObject, bool enabled)
        {
            dependencyObject.SetValue(IsEnabledProperty, enabled);
        }

        private static void OnIsEnabledPropertyChanged(DependencyObject dependencyObject, DependencyPropertyChangedEventArgs e)
        {
            RadGridView grid = dependencyObject as RadGridView;
            if (grid != null)
            {
                if ((bool) e.NewValue)
                {
                    RadGridViewSettings settings = new RadGridViewSettings(grid);
                    settings.Attach();

                    //if (grid.IsInitialized)
                    //{
                    //    settings.LoadState();
                    //}
                }
            }
        }

        public virtual void LoadState()
        {
            Settings.Reload();

            if (grid != null)
            {
                grid.Loaded += new RoutedEventHandler(grid_Loaded);

                grid.FrozenColumnCount = Settings.FrozenColumnCount;

                if (Settings.ColumnSettings.Count > 0)
                {
                    grid.AutoGenerateColumns = false;

                    foreach (ColumnSetting setting in Settings.ColumnSettings)
                    {
                        GridViewDataColumn column = new GridViewDataColumn();
                        column.UniqueName = setting.UniqueName;
                        column.HeaderText = setting.HeaderText;
                        if (setting.Width != null)
                        {
                            column.Width = new GridLength(setting.Width.Value);
                        }

                        grid.Columns.Add(column);
                    }
                }

                if (Settings.SortSettings.Count > 0)
                {
                    grid.SortDescriptions.Clear();

                    foreach (SortSetting setting in Settings.SortSettings)
                    {
                        SortDescription description = new SortDescription();
                        description.PropertyName = setting.PropertyName;
                        description.Direction = setting.SortDirection;

                        grid.SortDescriptions.Add(description);
                    }
                }

                if (Settings.GroupSettings.Count > 0)
                {
                    grid.GroupDescriptions.Clear();

                    foreach (GroupSetting setting in Settings.GroupSettings)
                    {
                        RadGroupDescription description = new RadGroupDescription();
                        description.PropertyName = setting.PropertyName;
                        description.SortDirection = setting.SortDirection;
                        description.DisplayContent = setting.DisplayContent;

                        grid.GroupDescriptions.Add(description);
                    }
                }
            }
        }

        void grid_Loaded(object sender, RoutedEventArgs e)
        {
            grid.Loaded -= new RoutedEventHandler(grid_Loaded);
            grid.ItemsControl.Loaded += new RoutedEventHandler(ItemsControl_Loaded);
        }

        void ItemsControl_Loaded(object sender, RoutedEventArgs e)
        {
            grid.ItemsControl.Loaded -= new RoutedEventHandler(ItemsControl_Loaded);
            if (Settings.FilterSettings.Count > 0)
            {
                CompositeFilterDescription cfd = grid.FilterDescription as CompositeFilterDescription;
                if (cfd != null)
                {
                    foreach (FilterSetting setting in Settings.FilterSettings)
                    {
                        FieldFilterDescription description = cfd.FilterDescriptions.OfType<FieldFilterDescription>().Where(fd => fd.FieldName == setting.PropertyName).FirstOrDefault();

                        if (description != null)
                        {
                            description.Filter1.Operator = setting.Filter1.Operator;
                            description.Filter1.Value = setting.Filter1.Value;

                            description.Filter2.Operator = setting.Filter2.Operator;
                            description.Filter2.Value = setting.Filter2.Value;

                            foreach (FilterDescriptor descriptor in setting.SelectedValues)
                            {
                                description.SelectedValues.Add(descriptor);
                            }
                        }
                    }
                }
            }
        }

        public virtual void ResetState()
        {
            Settings.Reset();
        }

        public virtual void SaveState()
        {
            if (grid != null)
            {
                if (grid.Columns != null)
                {
                    Settings.ColumnSettings.Clear();

                    foreach (GridViewColumn column in grid.Columns)
                    {
                        if (column is GridViewDataColumn)
                        {
                            GridViewDataColumn dataColumn = (GridViewDataColumn) column;

                            ColumnSetting setting = new ColumnSetting();
                            setting.PropertyName = dataColumn.DataMemberBinding.Path.Path;
                            setting.UniqueName = dataColumn.UniqueName;
                            setting.HeaderText = dataColumn.HeaderText;

                            GridViewHeaderRow headerRow = grid.ChildrenOfType<GridViewHeaderRow>().FirstOrDefault();
                            if (headerRow != null)
                            {
                                GridViewHeaderCell cell = headerRow.ChildrenOfType<GridViewHeaderCell>().Where(c => c.Column.UniqueName == setting.PropertyName).FirstOrDefault();
                                if (cell != null)
                                {
                                    setting.Width = cell.ActualWidth;
                                }
                            }

                            Settings.ColumnSettings.Add(setting);
                        }
                    }
                }

                if (grid.FilterDescription != null)
                {
                    Settings.FilterSettings.Clear();

                    CompositeFilterDescription cfd = grid.FilterDescription as CompositeFilterDescription;

                    foreach (FilterDescription description in cfd.FilterDescriptions)
                    {
                        if (description is FieldFilterDescription)
                        {
                            FieldFilterDescription ffd = (FieldFilterDescription) description;

                            if (ffd.IsActive)
                            {
                                FilterSetting setting = new FilterSetting();

                                setting.Filter1 = ffd.Filter1;
                                setting.Filter2 = ffd.Filter2;
                                setting.SelectedValues = new List<FilterDescriptor>(ffd.SelectedValues);
                                setting.PropertyName = ffd.FieldName;

                                Settings.FilterSettings.Add(setting);
                            }
                        }
                    }
                }

                if (grid.SortDescriptions != null)
                {
                    Settings.SortSettings.Clear();

                    foreach (SortDescription description in grid.SortDescriptions)
                    {
                        SortSetting setting = new SortSetting();

                        setting.PropertyName = description.PropertyName;
                        setting.SortDirection = description.Direction;

                        Settings.SortSettings.Add(setting);
                    }
                }

                if (grid.GroupDescriptions != null)
                {
                    Settings.GroupSettings.Clear();

                    foreach (RadGroupDescription description in grid.GroupDescriptions)
                    {
                        GroupSetting setting = new GroupSetting();

                        setting.PropertyName = description.PropertyName;
                        setting.SortDirection = description.SortDirection;
                        setting.DisplayContent = description.DisplayContent;

                        Settings.GroupSettings.Add(setting);
                    }
                }

                Settings.FrozenColumnCount = grid.FrozenColumnCount;
            }

            Settings.Save();
        }

        private void Attach()
        {
            if (grid != null)
            {
                grid.LayoutUpdated += new EventHandler(LayoutUpdated);
                grid.Loaded += new RoutedEventHandler(Loaded);
                Application.Current.Exit += new EventHandler(Current_Exit);
            }
        }

        void Current_Exit(object sender, EventArgs e)
        {
            SaveState();
        }

        void Loaded(object sender, RoutedEventArgs e)
        {
            LoadState();
        }

        void LayoutUpdated(object sender, EventArgs e)
        {
            if (grid.Parent == null)
            {
                SaveState();
            }
        }

        private RadGridViewApplicationSettings radGridViewApplicationSettings = null;

        protected virtual RadGridViewApplicationSettings CreateRadGridViewApplicationSettingsInstance()
        {
            return new RadGridViewApplicationSettings(this);
        }

        protected RadGridViewApplicationSettings Settings
        {
            get
            {
                if (radGridViewApplicationSettings == null)
                {
                    radGridViewApplicationSettings = CreateRadGridViewApplicationSettingsInstance();
                }
                return radGridViewApplicationSettings;
            }
        }
    }

    public class PropertySetting
    {
        public string PropertyName { get; set; }
    }

    public class SortSetting : PropertySetting
    {
        public ListSortDirection SortDirection { get; set; }
    }

    public class GroupSetting : SortSetting
    {
        public object DisplayContent { get; set; }
    }

    public class FilterSetting : PropertySetting
    {
        public List<FilterDescriptor> SelectedValues { get; set; }
        public FilterDescriptor Filter1 { get; set; }
        public FilterDescriptor Filter2 { get; set; }
    }

    public class ColumnSetting : PropertySetting
    {
        public string UniqueName { get; set; }
        public string HeaderText { get; set; }
        public double? Width { get; set; }
    }
}
